/* Input : output in the style of Unix find
   Output: same, but with a prepended creation and modification date,
           in the format "yyyymmdd-hhmmss yyyymmdd-hhmmss "
*/

/* 28 April 2011:
   - the same effect could probably also be achieved using stat(1)
   - Today I changed the st_ctime (change time), which is not very useful
     but which IS used to sort output in rsitemap, by st_birthtime.
     This is the birth date of the inode. At least this OS supports
     that. It is probably much more useful for what I really want
     to achieve with this program, i.e. the creation date, not the
     change date.

   5 Feb 2012:
     Send only one timestamp, not two. This can be the modification
     date (default -r argv[1] == -mtime) or the file's Birth date
     (FreeBSD specific): argv[1] = -Btime .

     rsitemap and sitemap.c have been changed for this too.

  28 March 2021:
     We now send the publication date-time (from the time-tag
     in the file), the modification date-time (from stat), and
     the language code (from the html-tage, attribute lang, in
     the file), then the path; field separated by semicolons (;).
     Date-time format is now yyyy-mm-ddThh:mm:ss .
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include "getfhtml.h"

int main (int argc, char **argv)
{
   static char path[1024];
   FILE *fpi = stdin;
   FILE *fpo = stdout;

   while (fgets(path, sizeof path, fpi))
   {
      char Btime[19+1];
      char mtime[19+1];
      struct stat st;
      char lang[3+1+2+1+4+1];
      char *p;
      struct tm *t;

      p = strrchr(path, '\n');
      if (p)
         *p = '\0';

      /* Stat to get file modification date-time. Skip non-existent files */
      if (stat(path, &st) != 0)
         continue;
      /* Accept only regular files, skip directories. */
      if (!(st.st_mode & S_IFREG))
         continue;
      t = localtime(&st.st_mtime);
      strftime(mtime, sizeof mtime, "%Y-%m-%dT%H:%M:%S", t);

      /* Get "birthtime"/publication date from the file */
      Btime[0] = '\0';
      p = gfh_GetFromPath(path, 1 << 13, "time", "datetime");
      strcpy(Btime, p ? p : "1970-01-01T12:00:00");
      if (Btime[10] != 'T')
         strcpy(Btime + 10, "T12:00:00");
      else if (Btime[13] != ':')
         strcpy(Btime + 13, ":00:00");
      else if (Btime[16] != ':')
         strcpy(Btime + 16, ":00");

      /* Get the language code (possibly with country and script)
         from the file */
      lang[0] = '\0';
      p = gfh_GetFromPath(path, 1 << 13, "html", "lang");
      if (p)
         strcpy(lang, p);

      fprintf(fpo, "%s;%s;%s;%s\n", Btime, mtime, lang, path);
   }

   return 0;
}
